from note_storage.cli.errors import exit_on_unsuccesful_response
from note_storage.gateways import SQLiteTagGateway
from note_storage.interactors import (
    GetTagCompletionsInteractor,
    GetTagsByFieldInteractor,
)
from note_storage.requests import (
    GetTagCompletionsRequest,
    GetTagsByFieldRequest,
)


async def complete_tag_name(db, incomplete, tag_names):
    tag_gateway = SQLiteTagGateway(db)

    interactor = GetTagsByFieldInteractor(tag_gateway)
    response = await interactor.execute(
        await GetTagsByFieldRequest.from_dict(
            {'field': 'name', 'values': tag_names}
        )
    )

    exit_on_unsuccesful_response(response)

    tag_ids = [tag.id for tag in response.value]

    interactor = GetTagCompletionsInteractor(tag_gateway)
    response = await interactor.execute(
        await GetTagCompletionsRequest.from_dict(
            {'incomplete': incomplete, 'completed_ids': tag_ids}
        )
    )

    exit_on_unsuccesful_response(response)

    return response.value
