import sys

from note_storage.cli.errors import exit_on_unsuccesful_response
from note_storage.gateways import SQLiteNoteGateway
from note_storage.interactors import (
    GetNotesWithTagsByFieldInteractor,
    UpdateNoteInteractor,
)
from note_storage.requests import (
    GetNotesWithTagsByFieldRequest,
    UpdateNoteRequest,
)


async def get_note_by_hex(db, hex_id):
    interactor = GetNotesWithTagsByFieldInteractor(SQLiteNoteGateway(db))

    response = await interactor.execute(
        await GetNotesWithTagsByFieldRequest.from_dict(
            {'field': 'hex_id', 'values': [hex_id]}
        )
    )

    exit_on_unsuccesful_response(response)

    notes_with_tags = response.value

    if not notes_with_tags:
        sys.exit(f'Note "{hex_id}" not found')

    return notes_with_tags[0]


async def update_note(db, note_id, note_ext, note_body, tag_names):
    interactor = UpdateNoteInteractor(SQLiteNoteGateway(db))

    response = await interactor.execute(
        await UpdateNoteRequest.from_dict(
            {
                'note_id': note_id,
                'note_ext': note_ext,
                'note_body': note_body,
                'tag_names': tag_names,
            }
        )
    )

    exit_on_unsuccesful_response(response)

    return response.value
