from note_storage.cli.errors import exit_on_unsuccesful_response
from note_storage.gateways import SQLiteNoteGateway, SQLiteTagGateway
from note_storage.interactors import (
    GetHexIDsByNoteIDsInteractor,
    GetNoteIdsByTagIdsInteractor,
    GetTagsByFieldInteractor,
)
from note_storage.requests import (
    GetHexIDsByNoteIDsRequest,
    GetNoteIdsByTagIdsRequest,
    GetTagsByFieldRequest,
)


async def get_hex_ids_by_note_ids(db, note_ids):
    interactor = GetHexIDsByNoteIDsInteractor(SQLiteNoteGateway(db))
    response = await interactor.execute(
        await GetHexIDsByNoteIDsRequest.from_dict({'note_ids': note_ids})
    )

    exit_on_unsuccesful_response(response)

    return response.value


async def search(db, tag_names):
    interactor = GetTagsByFieldInteractor(SQLiteTagGateway(db))
    response = await interactor.execute(
        await GetTagsByFieldRequest.from_dict(
            {'field': 'name', 'values': tag_names}
        )
    )

    exit_on_unsuccesful_response(response)

    tags = response.value

    interactor = GetNoteIdsByTagIdsInteractor(SQLiteNoteGateway(db))
    response = await interactor.execute(
        await GetNoteIdsByTagIdsRequest.from_dict(
            {'tag_ids': [tag.id for tag in tags]}
        )
    )

    exit_on_unsuccesful_response(response)

    return response.value
