import sys
from tempfile import NamedTemporaryFile

import click

from note_storage.core.constants import PROJECT_NAME
from note_storage.responses import ResponseFailure


def dump_error_to_file(response):
    exc_file = NamedTemporaryFile(
        mode='w', prefix=f'{PROJECT_NAME}_crash_', delete=False
    )

    exc_file.write(response.message + '\n')
    if response.exception:
        exc_file.write(response.exception)

    return exc_file.name


def exit_on_unsuccesful_response(response):
    if response.is_successful:
        return

    if response.type in (
        ResponseFailure.BAD_REQUEST,
        ResponseFailure.NOT_FOUND,
    ):
        click.echo(response.message, err=True)
        sys.exit(1)

    if response.type == ResponseFailure.INTERNAL_ERROR:
        filename = dump_error_to_file(response)
        error_message = (
            f'Unfortunately, {PROJECT_NAME} has crashed.\n'
            f'Error information was saved to file "{filename}".'
        )

        click.echo(error_message, err=True)
        sys.exit(1)
