from contextlib import asynccontextmanager
from pathlib import Path

import aiosqlite

from note_storage.core.config import config
from note_storage.core.db.schema_updates import db_updater
from note_storage.core.db.setup import prepare_db
from note_storage.core.db.utils import get_db_path


@asynccontextmanager
async def db_context():
    project_data_path = Path(config.get('general', 'data_path'))
    db_path = get_db_path(project_data_path)

    async with aiosqlite.connect(db_path) as db:
        await prepare_db(db)

        await db_updater.apply_updates(db)

        yield db
