from dataclasses import dataclass
from typing import List

from note_storage.core.db.utils import compute_schema_version


async def ensure_version_table_exists(db):
    async with db.execute(
        'SELECT name FROM sqlite_master WHERE '
        'type="table" AND name="db_version"'
    ) as cursor:
        version_table_exists = await cursor.fetchone() is not None

    if not version_table_exists:
        await db.execute('CREATE TABLE db_version ( version TEXT NOT NULL )')
        await db.execute(
            'INSERT INTO db_version VALUES ("{}")'.format(
                await compute_schema_version(db)
            )
        )
        await db.commit()


@dataclass
class SchemaUpdate:
    apply_to_version: str
    upd_recipes: List[str]


class DBUpdater:
    def __init__(self, updates):
        self.updates = updates

    @staticmethod
    async def get_current_version(db):
        async with db.execute('SELECT version FROM db_version') as cursor:
            return (await cursor.fetchone())[0]

    @staticmethod
    async def set_current_version(db, version):
        await db.execute(f'UPDATE db_version SET version = "{version}"')
        await db.commit()

    def next_schema_update(self, current_version):
        for update in self.updates:
            if update.apply_to_version == current_version:
                return update

        return None

    async def apply_updates(self, db):
        await ensure_version_table_exists(db)

        current_version = await self.get_current_version(db)

        while update := self.next_schema_update(current_version):
            for sql in update.upd_recipes:
                await db.execute(sql)

            current_version = await compute_schema_version(db)
            await self.set_current_version(db, current_version)


db_updater = DBUpdater(
    [
        SchemaUpdate(
            'c197c350f6e68dfc67300ccec20a4eed7ce5c932',
            [
                'ALTER TABLE notes ADD COLUMN '
                'extension TEXT NOT NULL DEFAULT "txt"'
            ],
        ),
        SchemaUpdate(
            '90fd3801d3802213a5e706e650f61dd7237600b7',
            [
                'CREATE TABLE system_storage '
                '( key TEXT NOT NULL UNIQUE, value TEXT NOT NULL )'
            ],
        ),
    ]
)
