import aiosqlite

CREATE_TABLES_SQL = [
    '''
    CREATE TABLE IF NOT EXISTS notes (
      id INTEGER PRIMARY KEY,
      hex_id TEXT NOT NULL UNIQUE,
      extension TEXT NOT NULL DEFAULT "txt",
      body TEXT NOT NULL
    )
    ''',
    '''
    CREATE TABLE IF NOT EXISTS tags (
      id INTEGER PRIMARY KEY,
      name TEXT NOT NULL UNIQUE
    )
    ''',
    '''
    CREATE TABLE IF NOT EXISTS note_tag (
      note_id INTEGER NOT NULL,
      tag_id INTEGER NOT NULL,
      UNIQUE(note_id, tag_id),
      FOREIGN KEY (note_id) REFERENCES notes(id) ON DELETE CASCADE,
      FOREIGN KEY (tag_id) REFERENCES tags(id) ON DELETE CASCADE
    )
    ''',
    '''
    CREATE TABLE IF NOT EXISTS system_storage (
      key TEXT NOT NULL UNIQUE,
      value TEXT NOT NULL
    )
    ''',
]


async def ensure_tables_exists(db):
    for sql in CREATE_TABLES_SQL:
        await db.execute(sql)


async def prepare_db(db):
    db.row_factory = aiosqlite.Row

    await db.execute('PRAGMA foreign_keys = ON')

    await ensure_tables_exists(db)
