from hashlib import sha1

from note_storage.core.constants import SQLITE_DB_NAME


def get_placeholders(number):
    return '(' + ', '.join(['?' for _ in range(number)]) + ')'


def get_db_path(project_data_path):
    if not project_data_path.exists():
        project_data_path.mkdir(parents=True)

    return project_data_path.joinpath(SQLITE_DB_NAME)


def raw_columns_schema_to_str(columns_info):
    return ', '.join(
        [
            ' '.join([str(x) for x in column_info])
            for column_info in columns_info
        ]
    )


async def get_table_names(db):
    async with db.execute(
        'SELECT name FROM sqlite_master WHERE '
        'type = "table" AND name != "db_version" ORDER BY name'
    ) as cursor:
        return [x[0] for x in await cursor.fetchall()]


async def get_columns_schema_for_table(db, table_name):
    async with db.execute(
        'SELECT name, type, "notnull", dflt_value FROM '
        f'pragma_table_info("{table_name}") ORDER BY name'
    ) as cursor:
        return raw_columns_schema_to_str(await cursor.fetchall())


async def compute_schema_version(db):
    table_names = await get_table_names(db)
    columns_schemas = [
        await get_columns_schema_for_table(db, name) for name in table_names
    ]

    db_schema = '\n'.join(
        [
            f'{name} ({schema})'
            for name, schema in zip(table_names, columns_schemas)
        ]
    )

    return sha1(db_schema.encode()).hexdigest()
