from note_storage.responses import ResponseFailure, ResponseSuccess


class AddTagsToNoteInteractor:
    def __init__(self, note_gateway, tag_gateway, note_id):
        self.note_gateway = note_gateway
        self.tag_gateway = tag_gateway
        self.note_id = note_id

    async def execute(self, request):
        if not request.is_valid:
            return ResponseFailure.build_from_invalid_request(request)

        gateway_args = (self.tag_gateway, self.note_id, request.tag_names)

        try:
            await self.note_gateway.link_tags_to_note(*gateway_args)
        except Exception as exc:
            message = f'Gateway was called with: {gateway_args!r}'
            return ResponseFailure.build_internal_error(message, exc)

        return ResponseSuccess()
