from note_storage.responses import ResponseFailure, ResponseSuccess


class CreateNoteInteractor:
    def __init__(self, gateway, generate_hex_id_func):
        self.gateway = gateway
        self.generate_hex_id_func = generate_hex_id_func

    async def execute(self, request):
        if not request.is_valid:
            return ResponseFailure.build_from_invalid_request(request)

        try:
            while True:
                hex_id = self.generate_hex_id_func()
                if await self.gateway.is_hex_id_available(hex_id):
                    break

            gateway_args = (
                hex_id,
                request.note_ext,
                request.note_body,
                request.tag_names,
            )

            await self.gateway.create_note(*gateway_args)
        except Exception as exc:
            message = f'Gateway was called with: {gateway_args!r}'
            return ResponseFailure.build_internal_error(message, exc)

        return ResponseSuccess(hex_id)
