from note_storage.responses import ResponseFailure, ResponseSuccess


class GetHexIDsByNoteIDsInteractor:
    def __init__(self, gateway):
        self.gateway = gateway

    async def execute(self, request):
        if not request.is_valid:
            return ResponseFailure.build_from_invalid_request(request)

        gateway_args = (request.note_ids,)

        try:
            hex_ids = await self.gateway.get_hex_ids_by_note_ids(*gateway_args)
        except Exception as exc:
            message = f'Gateway was called with: {gateway_args!r}'
            return ResponseFailure.build_internal_error(message, exc)

        return ResponseSuccess(hex_ids)
