from note_storage.responses import ResponseFailure, ResponseSuccess


class GetTagCompletionsInteractor:
    def __init__(self, gateway):
        self.gateway = gateway

    async def execute(self, request):
        if not request.is_valid:
            return ResponseFailure.build_from_invalid_request(request)

        gateway_args = (request.incomplete, request.completed_ids)

        try:
            tag_completions = await self.gateway.get_tag_completions(
                *gateway_args
            )
        except Exception as exc:
            message = f'Gateway was called with: {gateway_args!r}'
            return ResponseFailure.build_internal_error(message, exc)

        return ResponseSuccess(tag_completions)
