from note_storage.core.db.errors import SQLiteNotFoundError
from note_storage.responses import ResponseFailure, ResponseSuccess


class RemoveNoteInteractor:
    def __init__(self, gateway):
        self.gateway = gateway

    async def execute(self, request):
        if not request.is_valid:
            return ResponseFailure.build_from_invalid_request(request)

        gateway_args = (request.hex_id,)

        try:
            await self.gateway.remove_note(*gateway_args)
        except SQLiteNotFoundError as exc:
            return ResponseFailure.build_not_found_error(str(exc))

        except Exception as exc:
            message = f'Gateway was called with: {gateway_args!r}'
            return ResponseFailure.build_internal_error(message, exc)

        return ResponseSuccess()
