from note_storage.responses import ResponseFailure, ResponseSuccess


class RemoveTagsFromNoteInteractor:
    def __init__(self, note_gateway, tag_gateway, note_id):
        self.note_gateway = note_gateway
        self.tag_gateway = tag_gateway
        self.note_id = note_id

    async def execute(self, request):
        if not request.is_valid:
            return ResponseFailure.build_from_invalid_request(request)

        gateway_args = (self.note_id, request.tag_names)

        try:
            await self.note_gateway.unlink_tags_from_note(*gateway_args)
        except Exception as exc:
            message = f'Note gateway was called with: {gateway_args!r}'
            return ResponseFailure.build_internal_error(message, exc)

        try:
            await self.tag_gateway.remove_orphaned_tags()
        except Exception as exc:
            message = 'Tag gateway was called without arguments'
            return ResponseFailure.build_internal_error(message, exc)

        return ResponseSuccess()
