from note_storage.core.db.errors import SQLiteUniqueConstraintError
from note_storage.responses import ResponseFailure, ResponseSuccess


class RenameTagInteractor:
    def __init__(self, gateway):
        self.gateway = gateway

    async def execute(self, request):
        if not request.is_valid:
            return ResponseFailure.build_from_invalid_request(request)

        gateway_args = (request.old_name, request.new_name)

        try:
            await self.gateway.rename_tag(*gateway_args)
        except SQLiteUniqueConstraintError as exc:
            return ResponseFailure.build_bad_request_error(str(exc))

        except Exception as exc:
            message = f'Gateway was called with: {gateway_args!r}'
            return ResponseFailure.build_internal_error(message, exc)

        return ResponseSuccess()
