from jsonschema import Draft7Validator

from .base_request import ValidRequest
from .validation import validate_request

validator = Draft7Validator(
    {
        'type': 'object',
        'properties': {
            'field': {'enum': ['id', 'hex_id']},
            'values': {
                'type': 'array',
                'items': {'type': ['integer', 'string']},
            },
        },
        'required': ['field', 'values'],
    }
)

validator.check_schema(validator.schema)


class GetNotesWithTagsByFieldRequest(ValidRequest):
    def __init__(self, field, values):
        self.field = field
        self.values = values

    @classmethod
    async def from_dict(cls, dict_):
        if invalid_request := (await validate_request(validator, dict_)):
            return invalid_request

        return cls(dict_['field'], dict_['values'])
