from jsonschema import Draft7Validator

from .base_request import ValidRequest
from .validation import validate_request

validator = Draft7Validator(
    {
        'type': 'object',
        'properties': {
            'incomplete': {'type': 'string'},
            'completed_ids': {'type': 'array', 'items': {'type': 'integer'}},
        },
        'required': ['incomplete', 'completed_ids'],
    }
)

validator.check_schema(validator.schema)


class GetTagCompletionsRequest(ValidRequest):
    def __init__(self, incomplete, completed_ids):
        self.incomplete = incomplete
        self.completed_ids = completed_ids

    @classmethod
    async def from_dict(cls, dict_):
        if invalid_request := (await validate_request(validator, dict_)):
            return invalid_request

        return cls(dict_['incomplete'], dict_['completed_ids'])
