from jsonschema import Draft7Validator

from .base_request import ValidRequest
from .validation import validate_request

validator = Draft7Validator(
    {
        'type': 'object',
        'properties': {
            'note_id': {'type': 'integer'},
            'note_ext': {'type': 'string'},
            'note_body': {'type': 'string'},
            'tag_names': {'type': 'array', 'items': {'type': 'string'}},
        },
        'required': ['note_id', 'note_body', 'tag_names'],
    }
)

validator.check_schema(validator.schema)


class UpdateNoteRequest(ValidRequest):
    def __init__(self, note_id, note_ext, note_body, tag_names):
        self.note_id = note_id
        self.note_ext = note_ext
        self.note_body = note_body
        self.tag_names = tag_names

    @classmethod
    async def from_dict(cls, dict_):
        if invalid_request := (await validate_request(validator, dict_)):
            return invalid_request

        return cls(
            dict_['note_id'],
            dict_['note_ext'],
            dict_['note_body'],
            dict_['tag_names'],
        )
