import traceback


class ResponseSuccess:
    SUCCESS = 'SUCCESS'

    def __init__(self, value=None):
        self.type = self.SUCCESS
        self.value = value

    @property
    def is_successful(self):
        return True


class ResponseFailure:
    BAD_REQUEST = 'BAD_REQUEST'
    INTERNAL_ERROR = 'INTERNAL_ERROR'
    NOT_FOUND = 'NOT_FOUND'

    def __init__(self, type_, message, exception):
        self.type = type_
        self.message = message
        self.exception = traceback.format_exc() if exception else None

    @property
    def is_successful(self):
        return False

    @classmethod
    def build_from_invalid_request(cls, request):
        errors = [
            '{parameter}: "{message}"'.format_map(error)
            for error in request.errors
        ]
        message = ', '.join(errors)

        return cls.build_bad_request_error(message)

    @classmethod
    def build_bad_request_error(cls, message, exception=None):
        return cls(cls.BAD_REQUEST, message, exception)

    @classmethod
    def build_not_found_error(cls, message, exception=None):
        return cls(cls.NOT_FOUND, message, exception)

    @classmethod
    def build_internal_error(cls, message, exception=None):
        return cls(cls.INTERNAL_ERROR, message, exception)
